/*
	June 2006
	
	S. M. Nazrul Alam 
	smna@cs.cornell.edu
	Cornell University

This program shows how various space filling polyhedra, namely, Cube, Hexagonal Prism, Rhombic Dodecahedron and Truncated Octahedron 
fills the space. During execution of the program following keys can be used to see various polyhedra.

t	- Truncated Octahedral tesselation; shows nodes and the volume they cover. Faces of the truncated octahedrons are transparent and the edges are drawn as lines.
T	- Truncated Octahedral tesselation; faces are nontransparent; so nodes and inner truncated octahedrons can not be seen.
b	- Truncated Octahedral tesselation of a big 3D space.
  
r	- Rhombic Dodecahedral tesselation; shows nodes and the volume they cover. Faces of the rhombic dodecahedrons are transparent and the edges are drawn as lines.
R	- Rhombic Dodecahedral tesselation; faces are nontransparent; so nodes and inner rhombic dodecahedrons can not be seen.

h	- Hexagonal Prism tesselation; shows nodes and the volume they cover. Faces of hexagonal prisms are transparent and the edges are drawn as lines.
H	- Hexagonal Prism tesselation; faces are nontransparent; so nodes and inner hexagonal prisms can not be seen.

c	- Cube tesselation; shows nodes and the volume they cover. Faces of cubes are transparent and the edges are drawn as lines.
C	- Cube tesselation; faces are nontransparent; so nodes and inner cubes can not be seen.

Other useful keys:

m	- Toggle movement
x	- Increas Movement along positive x-axis
X	- Decrease Movement along negative x-axis
y	- Increas Movement along positive y-axis
Y	- Decrease Movement along negative y-axis
z	- Increas Movement along positive z-axis
Z	- Decrease Movement along negative z-axis
+	- Increase rotate speed
-	- Decrease rotate speed
Esc	- Terminate the program	
*/



#include <math.h>		
#include <stdlib.h>		
#include <GL/glut.h>	

#define sqrt3 1.7320508075688773
#define sqrt2 1.414213562373095
#define sqrt10 3.16227766016838


int mode=1; //Default TOWire
float x_angle = 0.0, y_angle = 0.0, z_angle = 0.0;
float rotate_speed = 0.6, step_speed = 2.0;
float view_distance = 50.0, cycle = 0.0;
int rotating =1, zooming = 0;


void draw(void);
void drawRDWire(void);
void drawRDSolid(void);
void drawTOWire(void);
void drawTOSolid(void);
void drawTOSolid2(void);
void drawCBWire(void);
void drawCBSolid(void);
void drawHPWire(void);
void drawHPSolid(void);


void idle(void);
void key(unsigned char k, int x, int y);
void reshape(int width, int height);
void init_drawing(void);

void draw_Truncated_Octahedron(float cx, float cy, float cz, float edge);
void draw_Truncated_Octahedron_Wire(float cx, float cy, float cz, float edge);
void draw_Rhombic_Dodecahedron(float cx, float cy, float cz, float radius);
void draw_Rhombic_Dodecahedron_Wire(float cx, float cy, float cz, float radius);
void draw_Hexagon_Wire(float hexcx, float hexcy, float hexcz, float hexR);
void draw_Hexagon(float hexcx, float hexcy, float hexcz, float hexR);
void draw_Cube(float cx, float cy, float cz, float edge);
void draw_Cube_Wire(float cx, float cy, float cz, float edge);
int main(int argc, char *argv[])
{

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutCreateWindow("Basic OpenGL Application");

	init_drawing();

	glutReshapeFunc(reshape);
	glutKeyboardFunc(key);
	glutIdleFunc(idle);
	glutDisplayFunc(draw);

	glutInitWindowSize(600, 600);
	glutReshapeWindow(600,600);

	glutMainLoop();

	return 0;             
}

void draw_Truncated_Octahedron(float cx, float cy, float cz, float edge)
{
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glEnable(GL_POLYGON_OFFSET_FILL);
	

//six square faces
	glColor3f(0.0, 0.5, 0.5);
	glBegin(GL_POLYGON);  // face 1 Top dir z
		glVertex3f(cx+edge/2, cy, cz+edge);
		glVertex3f(cx, cy+edge/2, cz+edge);
		glVertex3f(cx-edge/2, cy, cz+edge);
		glVertex3f(cx, cy-edge/2, cz+edge);
	glEnd();
	glColor3f(0.5, 0.0, 0.0);
	glBegin(GL_POLYGON);  // face 2 Bottom dir z
		glVertex3f(cx+edge/2, cy, cz-edge);
		glVertex3f(cx, cy+edge/2, cz-edge);
		glVertex3f(cx-edge/2, cy, cz-edge);
		glVertex3f(cx, cy-edge/2, cz-edge);
	glEnd();
	//glColor3f(0.5, 0.5, 0.0);
	glBegin(GL_POLYGON);  // face 3 Top dir x
		glVertex3f(cx+edge, cy+edge/2, cz);
		glVertex3f(cx+edge, cy, cz+edge/2);
		glVertex3f(cx+edge, cy-edge/2, cz);
		glVertex3f(cx+edge, cy, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 4 Bottom dir x
		glVertex3f(cx-edge, cy+edge/2, cz);
		glVertex3f(cx-edge, cy, cz+edge/2);
		glVertex3f(cx-edge, cy-edge/2, cz);
		glVertex3f(cx-edge, cy, cz-edge/2);
	glEnd();
	//glColor3f(0.5, 0.0, 0.5);
	glBegin(GL_POLYGON);  // face 5 Top dir y
		glVertex3f(cx+edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz+edge/2);
		glVertex3f(cx-edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz-edge/2);
	glEnd();
	
	
	glBegin(GL_POLYGON);  // face 6 Bottom dir y
		glVertex3f(cx+edge/2, cy-edge, cz);
		glVertex3f(cx, cy-edge, cz+edge/2);
		glVertex3f(cx-edge/2, cy-edge, cz);
		glVertex3f(cx, cy-edge, cz-edge/2);
	glEnd();

	

//8 hexagonal faces
	glColor3f(0.4, 0.4, 0.4);
	glBegin(GL_POLYGON);  // face 7 bottom z top x top y
		glVertex3f(cx, cy+edge/2, cz-edge);	
		glVertex3f(cx+edge/2, cy, cz-edge);
		glVertex3f(cx+edge, cy, cz-edge/2);
		glVertex3f(cx+edge, cy+edge/2, cz);
		glVertex3f(cx+edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz-edge/2);
	glEnd();

	glColor3f(0.7, 0.7, 0.7);
	glBegin(GL_POLYGON);  // face 8 bottom z bottom x top y
		glVertex3f(cx, cy+edge/2, cz-edge);
		glVertex3f(cx-edge/2, cy, cz-edge);
		glVertex3f(cx-edge, cy, cz-edge/2);
		glVertex3f(cx-edge, cy+edge/2, cz);
		glVertex3f(cx-edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz-edge/2);
	glEnd();

	glColor3f(0.4, 0.4, 0.4);
	glBegin(GL_POLYGON);  // face 9 bottom z bottom x bottom y
		glVertex3f(cx-edge/2, cy, cz-edge);
		glVertex3f(cx, cy-edge/2, cz-edge);
		glVertex3f(cx, cy-edge, cz-edge/2);
		glVertex3f(cx-edge/2, cy-edge, cz);
		glVertex3f(cx-edge, cy-edge/2, cz);
		glVertex3f(cx-edge, cy, cz-edge/2);
	glEnd();

	glColor3f(0.7, 0.7, 0.7);
	glBegin(GL_POLYGON);  // face 10 bottom z top x bottom y
		glVertex3f(cx+edge/2, cy, cz-edge);	
		glVertex3f(cx, cy-edge/2, cz-edge);	
		glVertex3f(cx, cy-edge, cz-edge/2);
		glVertex3f(cx+edge/2, cy-edge, cz);
		glVertex3f(cx+edge, cy-edge/2, cz);
		glVertex3f(cx+edge, cy, cz-edge/2);
	glEnd();
	glColor3f(0.4, 0.4, 0.4);
	glBegin(GL_POLYGON);  // face 11 top z top x top y
		glVertex3f(cx, cy+edge/2, cz+edge);	
		glVertex3f(cx+edge/2, cy, cz+edge);
		glVertex3f(cx+edge, cy, cz+edge/2);
		glVertex3f(cx+edge, cy+edge/2, cz);
		glVertex3f(cx+edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz+edge/2);
	glEnd();

	glColor3f(0.7, 0.7, 0.7);
	glBegin(GL_POLYGON);  // face 12 top z bottom x top y
		glVertex3f(cx, cy+edge/2, cz+edge);
		glVertex3f(cx-edge/2, cy, cz+edge);
		glVertex3f(cx-edge, cy, cz+edge/2);
		glVertex3f(cx-edge, cy+edge/2, cz);
		glVertex3f(cx-edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz+edge/2);
	glEnd();

	glColor3f(0.4, 0.4, 0.4);
	glBegin(GL_POLYGON);  // face 13 top z bottom x bottom y
		glVertex3f(cx-edge/2, cy, cz+edge);
		glVertex3f(cx, cy-edge/2, cz+edge);
		glVertex3f(cx, cy-edge, cz+edge/2);
		glVertex3f(cx-edge/2, cy-edge, cz);
		glVertex3f(cx-edge, cy-edge/2, cz);
		glVertex3f(cx-edge, cy, cz+edge/2);
	glEnd();

	glColor3f(0.7, 0.7, 0.7);
	glBegin(GL_POLYGON);  // face 14 top z top x bottom y
		glVertex3f(cx+edge/2, cy, cz+edge);	
		glVertex3f(cx, cy-edge/2, cz+edge);	
		glVertex3f(cx, cy-edge, cz+edge/2);
		glVertex3f(cx+edge/2, cy-edge, cz);
		glVertex3f(cx+edge, cy-edge/2, cz);
		glVertex3f(cx+edge, cy, cz+edge/2);
	glEnd();


	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glEnable(GL_POLYGON_OFFSET_LINE);
	glColor3f(0.0, 0.0, 0.0);
	glLineWidth(1.0);	
//six square faces
	glBegin(GL_POLYGON);  // face 1 Top dir z
		glVertex3f(cx+edge/2, cy, cz+edge);
		glVertex3f(cx, cy+edge/2, cz+edge);
		glVertex3f(cx-edge/2, cy, cz+edge);
		glVertex3f(cx, cy-edge/2, cz+edge);
	glEnd();
	
	glBegin(GL_POLYGON);  // face 2 Bottom dir z
		glVertex3f(cx+edge/2, cy, cz-edge);
		glVertex3f(cx, cy+edge/2, cz-edge);
		glVertex3f(cx-edge/2, cy, cz-edge);
		glVertex3f(cx, cy-edge/2, cz-edge);
	glEnd();
	
	glBegin(GL_POLYGON);  // face 3 Top dir x
		glVertex3f(cx+edge, cy+edge/2, cz);
		glVertex3f(cx+edge, cy, cz+edge/2);
		glVertex3f(cx+edge, cy-edge/2, cz);
		glVertex3f(cx+edge, cy, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 4 Bottom dir x
		glVertex3f(cx-edge, cy+edge/2, cz);
		glVertex3f(cx-edge, cy, cz+edge/2);
		glVertex3f(cx-edge, cy-edge/2, cz);
		glVertex3f(cx-edge, cy, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 5 Top dir y
		glVertex3f(cx+edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz+edge/2);
		glVertex3f(cx-edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz-edge/2);
	glEnd();
	
	glBegin(GL_POLYGON);  // face 6 Bottom dir y
		glVertex3f(cx+edge/2, cy-edge, cz);
		glVertex3f(cx, cy-edge, cz+edge/2);
		glVertex3f(cx-edge/2, cy-edge, cz);
		glVertex3f(cx, cy-edge, cz-edge/2);
	glEnd();


//8 hexagonal faces

	glBegin(GL_POLYGON);  // face 7 bottom z top x top y
		glVertex3f(cx, cy+edge/2, cz-edge);	
		glVertex3f(cx+edge/2, cy, cz-edge);
		glVertex3f(cx+edge, cy, cz-edge/2);
		glVertex3f(cx+edge, cy+edge/2, cz);
		glVertex3f(cx+edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 8 bottom z bottom x top y
		glVertex3f(cx, cy+edge/2, cz-edge);
		glVertex3f(cx-edge/2, cy, cz-edge);
		glVertex3f(cx-edge, cy, cz-edge/2);
		glVertex3f(cx-edge, cy+edge/2, cz);
		glVertex3f(cx-edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 9 bottom z bottom x bottom y
		glVertex3f(cx-edge/2, cy, cz-edge);
		glVertex3f(cx, cy-edge/2, cz-edge);
		glVertex3f(cx, cy-edge, cz-edge/2);
		glVertex3f(cx-edge/2, cy-edge, cz);
		glVertex3f(cx-edge, cy-edge/2, cz);
		glVertex3f(cx-edge, cy, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 10 bottom z top x bottom y
		glVertex3f(cx+edge/2, cy, cz-edge);	
		glVertex3f(cx, cy-edge/2, cz-edge);	
		glVertex3f(cx, cy-edge, cz-edge/2);
		glVertex3f(cx+edge/2, cy-edge, cz);
		glVertex3f(cx+edge, cy-edge/2, cz);
		glVertex3f(cx+edge, cy, cz-edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 11 top z top x top y
		glVertex3f(cx, cy+edge/2, cz+edge);	
		glVertex3f(cx+edge/2, cy, cz+edge);
		glVertex3f(cx+edge, cy, cz+edge/2);
		glVertex3f(cx+edge, cy+edge/2, cz);
		glVertex3f(cx+edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz+edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 12 top z bottom x top y
		glVertex3f(cx, cy+edge/2, cz+edge);
		glVertex3f(cx-edge/2, cy, cz+edge);
		glVertex3f(cx-edge, cy, cz+edge/2);
		glVertex3f(cx-edge, cy+edge/2, cz);
		glVertex3f(cx-edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz+edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 13 top z bottom x bottom y
		glVertex3f(cx-edge/2, cy, cz+edge);
		glVertex3f(cx, cy-edge/2, cz+edge);
		glVertex3f(cx, cy-edge, cz+edge/2);
		glVertex3f(cx-edge/2, cy-edge, cz);
		glVertex3f(cx-edge, cy-edge/2, cz);
		glVertex3f(cx-edge, cy, cz+edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 14 top z top x bottom y
		glVertex3f(cx+edge/2, cy, cz+edge);	
		glVertex3f(cx, cy-edge/2, cz+edge);	
		glVertex3f(cx, cy-edge, cz+edge/2);
		glVertex3f(cx+edge/2, cy-edge, cz);
		glVertex3f(cx+edge, cy-edge/2, cz);
		glVertex3f(cx+edge, cy, cz+edge/2);
	glEnd();

	
}

void draw_Truncated_Octahedron_Wire(float cx, float cy, float cz, float edge)
{

//draw edges of the surfaces
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glEnable(GL_POLYGON_OFFSET_LINE);
	//glColor3f(0.5, 0.5, 0.5);
	glLineWidth(1.1);	
//six square faces
	glBegin(GL_POLYGON);  // face 1 Top dir z
		glVertex3f(cx+edge/2, cy, cz+edge);
		glVertex3f(cx, cy+edge/2, cz+edge);
		glVertex3f(cx-edge/2, cy, cz+edge);
		glVertex3f(cx, cy-edge/2, cz+edge);
	glEnd();
	glBegin(GL_POLYGON);  // face 2 Bottom dir z
		glVertex3f(cx+edge/2, cy, cz-edge);
		glVertex3f(cx, cy+edge/2, cz-edge);
		glVertex3f(cx-edge/2, cy, cz-edge);
		glVertex3f(cx, cy-edge/2, cz-edge);
	glEnd();
	glBegin(GL_POLYGON);  // face 3 Top dir x
		glVertex3f(cx+edge, cy+edge/2, cz);
		glVertex3f(cx+edge, cy, cz+edge/2);
		glVertex3f(cx+edge, cy-edge/2, cz);
		glVertex3f(cx+edge, cy, cz-edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 4 Bottom dir x
		glVertex3f(cx-edge, cy+edge/2, cz);
		glVertex3f(cx-edge, cy, cz+edge/2);
		glVertex3f(cx-edge, cy-edge/2, cz);
		glVertex3f(cx-edge, cy, cz-edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 5 Top dir y
		glVertex3f(cx+edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz+edge/2);
		glVertex3f(cx-edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz-edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 6 Bottom dir y
		glVertex3f(cx+edge/2, cy-edge, cz);
		glVertex3f(cx, cy-edge, cz+edge/2);
		glVertex3f(cx-edge/2, cy-edge, cz);
		glVertex3f(cx, cy-edge, cz-edge/2);
	glEnd();
//8 hexagonal faces
	glBegin(GL_POLYGON);  // face 7 bottom z top x top y
		glVertex3f(cx, cy+edge/2, cz-edge);	
		glVertex3f(cx+edge/2, cy, cz-edge);
		glVertex3f(cx+edge, cy, cz-edge/2);
		glVertex3f(cx+edge, cy+edge/2, cz);
		glVertex3f(cx+edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz-edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 8 bottom z bottom x top y
		glVertex3f(cx, cy+edge/2, cz-edge);
		glVertex3f(cx-edge/2, cy, cz-edge);
		glVertex3f(cx-edge, cy, cz-edge/2);
		glVertex3f(cx-edge, cy+edge/2, cz);
		glVertex3f(cx-edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz-edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 9 bottom z bottom x bottom y
		glVertex3f(cx-edge/2, cy, cz-edge);
		glVertex3f(cx, cy-edge/2, cz-edge);
		glVertex3f(cx, cy-edge, cz-edge/2);
		glVertex3f(cx-edge/2, cy-edge, cz);
		glVertex3f(cx-edge, cy-edge/2, cz);
		glVertex3f(cx-edge, cy, cz-edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 10 bottom z top x bottom y
		glVertex3f(cx+edge/2, cy, cz-edge);	
		glVertex3f(cx, cy-edge/2, cz-edge);	
		glVertex3f(cx, cy-edge, cz-edge/2);
		glVertex3f(cx+edge/2, cy-edge, cz);
		glVertex3f(cx+edge, cy-edge/2, cz);
		glVertex3f(cx+edge, cy, cz-edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 11 top z top x top y
		glVertex3f(cx, cy+edge/2, cz+edge);	
		glVertex3f(cx+edge/2, cy, cz+edge);
		glVertex3f(cx+edge, cy, cz+edge/2);
		glVertex3f(cx+edge, cy+edge/2, cz);
		glVertex3f(cx+edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz+edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 12 top z bottom x top y
		glVertex3f(cx, cy+edge/2, cz+edge);
		glVertex3f(cx-edge/2, cy, cz+edge);
		glVertex3f(cx-edge, cy, cz+edge/2);
		glVertex3f(cx-edge, cy+edge/2, cz);
		glVertex3f(cx-edge/2, cy+edge, cz);
		glVertex3f(cx, cy+edge, cz+edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 13 top z bottom x bottom y
		glVertex3f(cx-edge/2, cy, cz+edge);
		glVertex3f(cx, cy-edge/2, cz+edge);
		glVertex3f(cx, cy-edge, cz+edge/2);
		glVertex3f(cx-edge/2, cy-edge, cz);
		glVertex3f(cx-edge, cy-edge/2, cz);
		glVertex3f(cx-edge, cy, cz+edge/2);
	glEnd();
	glBegin(GL_POLYGON);  // face 14 top z top x bottom y
		glVertex3f(cx+edge/2, cy, cz+edge);	
		glVertex3f(cx, cy-edge/2, cz+edge);	
		glVertex3f(cx, cy-edge, cz+edge/2);
		glVertex3f(cx+edge/2, cy-edge, cz);
		glVertex3f(cx+edge, cy-edge/2, cz);
		glVertex3f(cx+edge, cy, cz+edge/2);
	glEnd();
}


void draw(void)
{ 
	switch(mode)
	{
		case 1: 
			drawTOWire();
			break;
		case 2:
			drawTOSolid();
			break;
		case 3:
			drawRDWire();
			break;
		case 4:
			drawRDSolid();
			break;
		case 5:
			drawCBWire();
			break;
		case 6:
			drawCBSolid();
			break;
		case 7:
			drawHPWire();
			break;
		case 8:
			drawHPSolid();
			break;
		case 9:
			drawTOSolid2();
			break;


	}
}


void drawTOWire(void)
{

	float T,R,edge;
	int x,y,z;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0, 0.0, -view_distance);
	glRotatef(x_angle, 1.0, 0.0, 0.0);
	glRotatef(y_angle, 0.0, 1.0, 0.0);
	glRotatef(z_angle, 0.0, 0.0, 1.0);


	glClearColor(1.0,1.0,1.0, 0.0);

	glPointSize(5.0);
    glColor3f(0.0, 0.0, 0.0);

	T=20;
	R=5;
	edge=(2*R*sqrt2)/sqrt10;
    glBegin(GL_POINTS);
		x=-30;
		while (x <= T)
		{
			y=-T;
			while (y <= T)
			{
				z=-T;
				while (z <=T)
				{
					if ( fabs((2*x+z)*edge)<=T && fabs((2*y+z)*edge) <=T && fabs(z*edge) <=T)
					{
						glVertex3f((2*x+z)*edge,(2*y+z)*edge,z*edge);
					}
					z++;
				}
				y++;
			}
		x++;
		}
    glEnd();



	glLineWidth(2.0);	

	glBegin(GL_LINES);
		glColor3f(0.3, 0.0, 0.0);
		glVertex3f(-T-3,0.0,0.0);
		glVertex3f(T+3, 0.0, 0.0);
		glColor3f(0.0, 0.3, 0.0);
		glVertex3f(0.0, -T-3,0.0);
		glVertex3f(0.0, T+3, 0.0);
		glColor3f(0.0, 0.0, 0.3);
		glVertex3f(0.0,0.0, -T-3.0);
		glVertex3f(0.0, 0.0, T+3);

	glEnd();
	x=-30;
		while (x <= T)
		{
			y=-T;
			while (y <= T)
			{
				z=-T;
				while (z <=T)
				{
					if ( fabs((2*x+z)*edge)<=T && fabs((2*y+z)*edge) <=T && fabs(z*edge) <=T)
					{
						glColor3f(0.1,0.1,0.1);
						draw_Truncated_Octahedron_Wire((2*x+z)*edge,(2*y+z)*edge,z*edge,edge);
					}
					z++;
				}
				y++;
			}
		x++;
		}

	glFlush();
	glutSwapBuffers();
}


void drawTOSolid(void)
{

	float T,R,edge;
	int x,y,z;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0, 0.0, -view_distance);
	glRotatef(x_angle, 1.0, 0.0, 0.0);
	glRotatef(y_angle, 0.0, 1.0, 0.0);
	glRotatef(z_angle, 0.0, 0.0, 1.0);


	glClearColor(1.0,1.0,1.0, 0.0);

	T=20;
	R=5;
	edge=(2*R*sqrt2)/sqrt10;
    

	x=-30;
		while (x <= T)
		{
			y=-T;
			while (y <= T)
			{
				z=-T;
				while (z <=T)
				{
					if ( fabs((2*x+z)*edge)<=T && fabs((2*y+z)*edge) <=T && fabs(z*edge) <=T)
					{
						glColor3f(0.1,0.1,0.1);
						draw_Truncated_Octahedron((2*x+z)*edge,(2*y+z)*edge,z*edge,edge);
					}
					z++;
				}
				y++;
			}
		x++;
		}

	glFlush();
	glutSwapBuffers();
}




void drawTOSolid2(void)
{

	float R,edge;
	int x,y,z;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -view_distance);
	glRotatef(x_angle, 1.0, 0.0, 0.0);
	glRotatef(y_angle, 0.0, 1.0, 0.0);
	glRotatef(z_angle, 0.0, 0.0, 1.0);


	glClearColor(1.0,1.0,1.0, 0.0);

	R=3;
	edge=(2*R*sqrt2)/sqrt10;

	for (z=-10;z<10;z++)
		for (x=-10;x<10;x++)
			for (y=-10;y<10;y++)
				draw_Truncated_Octahedron(20+(2*x+z)*edge,20+(2*y+z)*edge,-50-z*edge,edge);

	glFlush();
	glutSwapBuffers();
}


void idle(void)
{
	if(rotating)
	{
		x_angle += rotate_speed;
		y_angle += rotate_speed;
		z_angle += rotate_speed;
		glutPostRedisplay();
	}

}

void key(unsigned char k, int x, int y)
{
	switch(k)
	{
	case 27: 
		exit(0);
		break;
	case 't':	//TOWire
		mode = 1;
		draw();
		break;
	case 'T':	//TOSolid
		mode = 2;
		draw();
		break;
	case 'r':	//RDWire
		mode = 3;
		draw();
		break;
	case 'R':	//RDSolid
		mode = 4;
		draw();
		break;
	case 'c':	//CBWire
		mode = 5;
		draw();
		break;
	case 'C':	//CBSolid
		mode = 6;
		draw();
		break;
	case 'h':	//HPWire
		mode = 7;
		draw();
		break;
	case 'H':	//HPSolid
		mode = 8;
		draw();
		break;
	case 'b':	//TOSolidbig
		mode = 9;
		draw();
		break;

	case 'm':
		rotating = 1-rotating;
		draw();
		break;
	case 'x':
		x_angle += step_speed;
		draw();
		break;
	case 'X':
		x_angle -= step_speed;
		draw();
		break;
	case 'y':
		y_angle += step_speed;
		draw();
		break;
	case 'Y':
		y_angle -= step_speed;
		draw();
		break;
	case 'z':
		z_angle += step_speed;
		draw();
		break;
	case 'Z':
		z_angle -= step_speed;
		draw();
		break;
	case '+':
		rotate_speed += 0.1;
		break;
	case '-':
		rotate_speed -= 0.1;
		break;
	}
}

void reshape(int width, int height)
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(90.0, (float)width/(float)height, 1.0, 100.0);
	draw();
}


void init_drawing(void)
{
	glShadeModel(GL_SMOOTH);
	glDisable(GL_LIGHTING);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);

}


void visible(int vis)
{
	if (vis == GLUT_VISIBLE)
		glutIdleFunc(idle);
	else
		glutIdleFunc(NULL);
}







void draw_Rhombic_Dodecahedron(float cx, float cy, float cz, float radius)
{	
	

	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glEnable(GL_POLYGON_OFFSET_FILL);
	

	glColor3f(0.7, 0.7, 0.7);
	glBegin(GL_POLYGON);  // face 1 dir +x
		glVertex3f(cx+radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx+radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
	glEnd();
	
	//glColor3f(0.8, 0.0, 0.8);
	glBegin(GL_POLYGON);  // face 2 dir -x
		glVertex3f(cx-radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx-radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
	glEnd();

	//glColor3f(0.8, 0.5, 0.7);
	glBegin(GL_POLYGON);  // face 3 dir +y
		glVertex3f(cx,cy+radius/sqrt2, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
	glEnd();
	//glColor3f(0.6, 0.6, 0.9);
	glBegin(GL_POLYGON);  // face 4 dir -y
		glVertex3f(cx,cy-radius/sqrt2, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
	glEnd();

	glColor3f(0.7, 0.7, 0.0);
	glBegin(GL_POLYGON);  // face 5 dir +z +x +y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx+radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz+radius/2);
	glEnd();

	glColor3f(0.0, 0.7, 0.7);
	glBegin(GL_POLYGON);  // face 6 dir +z -x +y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx-radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz+radius/2);
	glEnd();

	glColor3f(0.0, 0.7, 0.7);
	glBegin(GL_POLYGON);  // face 7 dir +z  +x  -y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx+radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz+radius/2);
	glEnd();

	glColor3f(0.7, 0.7, 0.0);
	glBegin(GL_POLYGON);  // face 8 dir +z -x -y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx-radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz+radius/2);
	glEnd();

	glColor3f(0.7, 0.7, 0.0);
	glBegin(GL_POLYGON);  // face 9 dir -z +x +y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx+radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz-radius/2);
	glEnd();

	
	glColor3f(0.0, 0.7, 0.7);
	glBegin(GL_POLYGON);  // face 10 dir -z -x +y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx-radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz-radius/2);
	glEnd();

	glColor3f(0.0, 0.7, 0.7);
	glBegin(GL_POLYGON);  // face 11 dir -z  +x  -y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx+radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz-radius/2);
	glEnd();

	glColor3f(0.7, 0.7, 0.0);
	glBegin(GL_POLYGON);  // face 12 dir -z -x -y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx-radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz-radius/2);
	glEnd();

	
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glEnable(GL_POLYGON_OFFSET_LINE);
	glColor3f(0.0, 0.0, 0.0);
	glLineWidth(2.0);	
	glBegin(GL_POLYGON);  // face 1 dir +x
		glVertex3f(cx+radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx+radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
	glEnd();
	
	glBegin(GL_POLYGON);  // face 2 dir -x
		glVertex3f(cx-radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx-radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
	glEnd();

	glBegin(GL_POLYGON);  // face 3 dir +y
		glVertex3f(cx,cy+radius/sqrt2, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
	glEnd();
	glBegin(GL_POLYGON);  // face 4 dir -y
		glVertex3f(cx,cy-radius/sqrt2, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
	glEnd();

	glBegin(GL_POLYGON);  // face 5 dir +z +x +y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx+radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz+radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 6 dir +z -x +y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx-radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz+radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 7 dir +z  +x  -y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx+radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz+radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 8 dir +z -x -y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx-radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz+radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 9 dir -z +x +y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx+radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz-radius/2);
	glEnd();

	
	glBegin(GL_POLYGON);  // face 10 dir -z -x +y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx-radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz-radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 11 dir -z  +x  -y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx+radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz-radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 12 dir -z -x -y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx-radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz-radius/2);
	glEnd();


}	

void draw_Rhombic_Dodecahedron_Wire(float cx, float cy, float cz, float radius)
{	
	

		
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glEnable(GL_POLYGON_OFFSET_LINE);
	//glColor3f(0.0, 0.0, 0.0);
	glLineWidth(1.0);	
	glBegin(GL_POLYGON);  // face 1 dir +x
		glVertex3f(cx+radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx+radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
	glEnd();
	
	glBegin(GL_POLYGON);  // face 2 dir -x
		glVertex3f(cx-radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx-radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
	glEnd();

	glBegin(GL_POLYGON);  // face 3 dir +y
		glVertex3f(cx,cy+radius/sqrt2, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
	glEnd();
	glBegin(GL_POLYGON);  // face 4 dir -y
		glVertex3f(cx,cy-radius/sqrt2, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
	glEnd();

	glBegin(GL_POLYGON);  // face 5 dir +z +x +y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx+radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz+radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 6 dir +z -x +y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx-radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz+radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 7 dir +z  +x  -y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx+radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz+radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 8 dir +z -x -y
		glVertex3f(cx,cy, cz+radius);
		glVertex3f(cx-radius/sqrt2,cy, cz+radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz+radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 9 dir -z +x +y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx+radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx+radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz-radius/2);
	glEnd();

	
	glBegin(GL_POLYGON);  // face 10 dir -z -x +y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx-radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy+radius/sqrt2, cz);
		glVertex3f(cx,cy+radius/sqrt2, cz-radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 11 dir -z  +x  -y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx+radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx+radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz-radius/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 12 dir -z -x -y
		glVertex3f(cx,cy, cz-radius);
		glVertex3f(cx-radius/sqrt2,cy, cz-radius/2);
		glVertex3f(cx-radius/sqrt2,cy-radius/sqrt2, cz);
		glVertex3f(cx,cy-radius/sqrt2, cz-radius/2);
	glEnd();


}	


void drawRDWire(void)
{ 
	float T=20;
	float R=5;
	int x,y,z;
	float edge;
	
	edge=(2*R*sqrt2)/sqrt10;
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0, 0.0, -view_distance);
	glRotatef(x_angle, 1.0, 0.0, 0.0);
	glRotatef(y_angle, 0.0, 1.0, 0.0);
	glRotatef(z_angle, 0.0, 0.0, 1.0);


	glClearColor(1.0,1.0,1.0, 0.0);
	glPointSize(5.0);
    glColor3f(0.0, 0.0, 0.0);
	
    glBegin(GL_POINTS);
		x=-30;
		while (x <= T)
		{
			y=-T;
			while (y <= T)
			{
				z=-T;
				while (z <=T)
				{
					if ( fabs(x*R*sqrt2+(abs(z)%2)*R/sqrt2)<=T && fabs(R*y*sqrt2+(abs(z)%2)*R/sqrt2) <=T && fabs(z*R) <=T)
					{
						glVertex3f(x*R*sqrt2+(abs(z)%2)*R/sqrt2,R*y*sqrt2+(abs(z)%2)*R/sqrt2,z*R);
						
					}
					z++;
				}
				y++;
			}
		x++;
		}
    glEnd();



	glLineWidth(2.0);	

	glBegin(GL_LINES);
	glColor3f(0.3, 0.0, 0.0);
	glVertex3f(-T-3,0.0,0.0);
	glVertex3f(T+3, 0.0, 0.0);
	glColor3f(0.0, 0.3, 0.0);
	glVertex3f(0.0, -T-3,0.0);
	glVertex3f(0.0, T+3, 0.0);
	glColor3f(0.0, 0.0, 0.3);
	glVertex3f(0.0,0.0, -T-3.0);
	glVertex3f(0.0, 0.0, T+3);

	glEnd();
	x=-30;
	while (x <= T)
		{
			y=-T;
			while (y <= T)
			{
				z=-T;
				while (z <=T)
				{
					if ( fabs(x*R*sqrt2+(abs(z)%2)*R/sqrt2)<=T && fabs(R*y*sqrt2+(abs(z)%2)*R/sqrt2) <=T && fabs(z*R) <=T)
					{
						glColor3f(0.5,0.5,0.5);
						draw_Rhombic_Dodecahedron_Wire(x*R*sqrt2+(abs(z)%2)*R/sqrt2,R*y*sqrt2+(abs(z)%2)*R/sqrt2,z*R,R);
					}
					z++;
				}
				y++;
			}
		x++;
		}


	/*float R=4;
	
	int u,v,w;
	w=0;v=0;u=0;
		for (w=-4;w<4;w++)
		for (u=-4;u<4;u++)
			for (v=-4;v<4;v++)
					draw_Rhombic_Dodecahedron(u*R*sqrt2+(abs(w)%2)*R/sqrt2,R*v*sqrt2+(abs(w)%2)*R/sqrt2,w*R,R);
		
*/
	glFlush();
	glutSwapBuffers();
}

void drawRDSolid(void)
{
	float T=20;
	float R=5;
	int x,y,z;
	float edge;
	
	edge=(2*R*sqrt2)/sqrt10;
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0, 0.0, -view_distance);
	glRotatef(x_angle, 1.0, 0.0, 0.0);
	glRotatef(y_angle, 0.0, 1.0, 0.0);
	glRotatef(z_angle, 0.0, 0.0, 1.0);


	glClearColor(1.0,1.0,1.0, 0.0);
	x=-30;
	while (x <= T)
		{
			y=-T;
			while (y <= T)
			{
				z=-T;
				while (z <=T)
				{
					if ( fabs(x*R*sqrt2+(abs(z)%2)*R/sqrt2)<=T && fabs(R*y*sqrt2+(abs(z)%2)*R/sqrt2) <=T && fabs(z*R) <=T)
					{
						glColor3f(0.5,0.5,0.5);
						draw_Rhombic_Dodecahedron(x*R*sqrt2+(abs(z)%2)*R/sqrt2,R*y*sqrt2+(abs(z)%2)*R/sqrt2,z*R,R);
					}
					z++;
				}
				y++;
			}
		x++;
		}


	glFlush();
	glutSwapBuffers();

}



void draw_Hexagon(float hexcx, float hexcy, float hexcz, float hexR)
{
	float hexc=hexR/sqrt3;
	float hexb=hexR/sqrt2;
	float hexa=hexc*sqrt2;
	float hexd=hexa/2;

	//(-a,0,c), (-d,b,c), (d,b,c), (a,0,c),(d,-b,c) ,(-d,-b,c)


	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glEnable(GL_POLYGON_OFFSET_FILL);
	glColor3f(0.0, 0.5, 0.5);


	glBegin(GL_POLYGON);  // face 1 Top
		glVertex3f(hexcx-hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz+hexc);

		glVertex3f(hexcx+hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz+hexc);
	glEnd();

	glBegin(GL_POLYGON);  // face 2 Bottom
		glVertex3f(hexcx-hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz-hexc);

		glVertex3f(hexcx+hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz-hexc);
	glEnd();

	glColor3f(0.5, 0.3, 0.3);
	//(-a,0,c), (-d,b,c), (-d,b,-c), (-a,0,-c)
	glBegin(GL_POLYGON);  // face 3
		glVertex3f(hexcx-hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz-hexc);
		glVertex3f(hexcx-hexa, hexcy, hexcz-hexc);
	glEnd();

	//(-d,b,c), (d,b,c), (d,b,-c), (-d,b,-c)
	glBegin(GL_POLYGON);  // face 4
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz-hexc);
	glEnd();

	 //(d,b,c), (a,0,c), (a,0,-c), (d,b,-c)
	glBegin(GL_POLYGON);  // face 5
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx+hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz-hexc);
	glEnd();

	//(a,0,c),(d,-b,c) , (d,-b,-c) , (a,0,-c)
	glBegin(GL_POLYGON);  // face 6
		glVertex3f(hexcx+hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz-hexc);
		glVertex3f(hexcx+hexa, hexcy, hexcz-hexc);
	glEnd();

	//(d,-b,c) ,(-d,-b,-c), (-d,-b,c), (d,-b,-c)
	glBegin(GL_POLYGON);  // face 7
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz-hexc);
	glEnd();

	//(-d,-b,c), (-a,0,c), (-a,0,-c), (-d,-b,-c)
	glBegin(GL_POLYGON);  // face 8
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx-hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz-hexc);
	glEnd();


	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glEnable(GL_POLYGON_OFFSET_LINE);
	glColor3f(0.0, 0.0, 0.0);
	glLineWidth(2.0);	

	glBegin(GL_POLYGON);  // face 1 Top
		glVertex3f(hexcx-hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz+hexc);

		glVertex3f(hexcx+hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz+hexc);
	glEnd();

	glBegin(GL_POLYGON);  // face 2 Bottom
		glVertex3f(hexcx-hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz-hexc);

		glVertex3f(hexcx+hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz-hexc);
	glEnd();

	//(-a,0,c), (-d,b,c), (-d,b,-c), (-a,0,-c)
	glBegin(GL_POLYGON);  // face 3
		glVertex3f(hexcx-hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz-hexc);
		glVertex3f(hexcx-hexa, hexcy, hexcz-hexc);
	glEnd();

	//(-d,b,c), (d,b,c), (d,b,-c), (-d,b,-c)
	glBegin(GL_POLYGON);  // face 4
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz-hexc);
	glEnd();

	 //(d,b,c), (a,0,c), (a,0,-c), (d,b,-c)
	glBegin(GL_POLYGON);  // face 5
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx+hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz-hexc);
	glEnd();

	//(a,0,c),(d,-b,c) , (d,-b,-c) , (a,0,-c)
	glBegin(GL_POLYGON);  // face 6
		glVertex3f(hexcx+hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz-hexc);
		glVertex3f(hexcx+hexa, hexcy, hexcz-hexc);
	glEnd();

	//(d,-b,c) ,(-d,-b,-c), (-d,-b,c), (d,-b,-c)
	glBegin(GL_POLYGON);  // face 7
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz-hexc);
	glEnd();

	//(-d,-b,c), (-a,0,c), (-a,0,-c), (-d,-b,-c)
	glBegin(GL_POLYGON);  // face 8
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx-hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz-hexc);
	glEnd();
}

void draw_Hexagon_Wire(float hexcx, float hexcy, float hexcz, float hexR)
{
	float hexc=hexR/sqrt3;
	float hexb=hexR/sqrt2;
	float hexa=hexc*sqrt2;
	float hexd=hexa/2;

	
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glEnable(GL_POLYGON_OFFSET_LINE);
	glColor3f(0.1, 0.1, 0.1);
	glLineWidth(1.4);	

	glBegin(GL_POLYGON);  // face 1 Top
		glVertex3f(hexcx-hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz+hexc);

		glVertex3f(hexcx+hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz+hexc);
	glEnd();

	glBegin(GL_POLYGON);  // face 2 Bottom
		glVertex3f(hexcx-hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz-hexc);

		glVertex3f(hexcx+hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz-hexc);
	glEnd();

	//(-a,0,c), (-d,b,c), (-d,b,-c), (-a,0,-c)
	glBegin(GL_POLYGON);  // face 3
		glVertex3f(hexcx-hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz-hexc);
		glVertex3f(hexcx-hexa, hexcy, hexcz-hexc);
	glEnd();

	//(-d,b,c), (d,b,c), (d,b,-c), (-d,b,-c)
	glBegin(GL_POLYGON);  // face 4
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy+hexb, hexcz-hexc);
	glEnd();

	 //(d,b,c), (a,0,c), (a,0,-c), (d,b,-c)
	glBegin(GL_POLYGON);  // face 5
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz+hexc);
		glVertex3f(hexcx+hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx+hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy+hexb, hexcz-hexc);
	glEnd();

	//(a,0,c),(d,-b,c) , (d,-b,-c) , (a,0,-c)
	glBegin(GL_POLYGON);  // face 6
		glVertex3f(hexcx+hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz-hexc);
		glVertex3f(hexcx+hexa, hexcy, hexcz-hexc);
	glEnd();

	//(d,-b,c) ,(-d,-b,-c), (-d,-b,c), (d,-b,-c)
	glBegin(GL_POLYGON);  // face 7
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz-hexc);
		glVertex3f(hexcx+hexd, hexcy-hexb, hexcz-hexc);
	glEnd();

	//(-d,-b,c), (-a,0,c), (-a,0,-c), (-d,-b,-c)
	glBegin(GL_POLYGON);  // face 8
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz+hexc);
		glVertex3f(hexcx-hexa, hexcy, hexcz+hexc);
		glVertex3f(hexcx-hexa, hexcy, hexcz-hexc);
		glVertex3f(hexcx-hexd, hexcy-hexb, hexcz-hexc);
	glEnd();
}

void drawHPWire(void)
{ 
	
	float T, hexR;
	int x,y,z;
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0, 0.0, -view_distance);
	glRotatef(x_angle, 1.0, 0.0, 0.0);
	glRotatef(y_angle, 0.0, 1.0, 0.0);
	glRotatef(z_angle, 0.0, 0.0, 1.0);


	glClearColor(1.0,1.0,1.0, 0.0);
	
	T=20;

	glLineWidth(2.0);	

	glBegin(GL_LINES);
	glColor3f(0.3, 0.0, 0.0);
		glVertex3f(-T-3,0.0,0.0);
		glVertex3f(T+3, 0.0, 0.0);
		glColor3f(0.0, 0.3, 0.0);
		glVertex3f(0.0, -T-3,0.0);
		glVertex3f(0.0, T+3, 0.0);
		glColor3f(0.0, 0.0, 0.3);
		glVertex3f(0.0,0.0, -T-3.0);
		glVertex3f(0.0, 0.0, T+3);

	glEnd();

	hexR=6;
	
	glPointSize(5.0);
    glColor3f(0.0, 0.0, 0.0);
    glBegin(GL_POINTS);
	for (z=-2;z<2;z++)
		for (x=-2;x<2;x++)
			for (y=-2;y<3;y++)
					glVertex3f((sqrt3*hexR*x)/sqrt2,((x+2*y)*hexR)/sqrt2,(2*hexR*z)/sqrt3);
	glEnd();

	for (z=-2;z<2;z++)
		for (x=-2;x<2;x++)
			for (y=-2;y<3;y++)
				draw_Hexagon_Wire((sqrt3*hexR*x)/sqrt2,((x+2*y)*hexR)/sqrt2,(2*hexR*z)/sqrt3,hexR);
			
	glFlush();
	glutSwapBuffers();
}


void drawHPSolid(void)
{ 
	
	float T, hexR;
	int x,y,z;
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0, 0.0, -view_distance);
	glRotatef(x_angle, 1.0, 0.0, 0.0);
	glRotatef(y_angle, 0.0, 1.0, 0.0);
	glRotatef(z_angle, 0.0, 0.0, 1.0);


	glClearColor(1.0,1.0,1.0, 0.0);
	
	T=20;

	hexR=6;
	
	
	for (z=-2;z<2;z++)
		for (x=-2;x<2;x++)
			for (y=-2;y<3;y++)
				draw_Hexagon((sqrt3*hexR*x)/sqrt2,((x+2*y)*hexR)/sqrt2,(2*hexR*z)/sqrt3,hexR);
			
	glFlush();
	glutSwapBuffers();
}

void draw_Cube(float cx, float cy, float cz, float edge)
{	
	
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glEnable(GL_POLYGON_OFFSET_FILL);
	
	glColor3f(0.0, 0.5, 0.5);
	glBegin(GL_POLYGON);  // face 1 dir +z
		glVertex3f(cx+edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz+edge/2);
	glEnd();

	glColor3f(0.5, 0.0, 0.5);
	glBegin(GL_POLYGON);  // face 2 dir -z
		glVertex3f(cx+edge/2,cy+edge/2, cz-edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz-edge/2);
	glEnd();
	
	glColor3f(0.5, 0.5, 0.0);
	glBegin(GL_POLYGON);  // face 3 dir +x
		glVertex3f(cx+edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx+edge/2,cy+edge/2, cz-edge/2);
	glEnd();

	glColor3f(0.3, 0.3,0.0);
	glBegin(GL_POLYGON);  // face 4 dir -x
		glVertex3f(cx-edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz-edge/2);
	glEnd();

	glColor3f(0.0, 0.3,0.3);
	glBegin(GL_POLYGON);  // face 5 dir +y
		glVertex3f(cx+edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy+edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz+edge/2);
	glEnd();

	glColor3f(0.3, 0.0,0.3);
	glBegin(GL_POLYGON);  // face 6 dir -y
		glVertex3f(cx+edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz+edge/2);
	glEnd();

	
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glEnable(GL_POLYGON_OFFSET_LINE);
	glColor3f(0.0, 0.0, 0.0);
	glLineWidth(2.0);	
	glBegin(GL_POLYGON);  // face 1 dir +z
		glVertex3f(cx+edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz+edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 2 dir -z
		glVertex3f(cx+edge/2,cy+edge/2, cz-edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz-edge/2);
	glEnd();
	
	glBegin(GL_POLYGON);  // face 3 dir +x
		glVertex3f(cx+edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx+edge/2,cy+edge/2, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 4 dir -x
		glVertex3f(cx-edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 5 dir +y
		glVertex3f(cx+edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy+edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz+edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 6 dir -y
		glVertex3f(cx+edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz+edge/2);
	glEnd();


}	

void draw_Cube_Wire(float cx, float cy, float cz, float edge)
{	
	
		
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glEnable(GL_POLYGON_OFFSET_LINE);
	glColor3f(0.1, 0.1, 0.1);
	glLineWidth(1.4);	
	glBegin(GL_POLYGON);  // face 1 dir +z
		glVertex3f(cx+edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz+edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 2 dir -z
		glVertex3f(cx+edge/2,cy+edge/2, cz-edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz-edge/2);
	glEnd();
	
	glBegin(GL_POLYGON);  // face 3 dir +x
		glVertex3f(cx+edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx+edge/2,cy+edge/2, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 4 dir -x
		glVertex3f(cx-edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz-edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 5 dir +y
		glVertex3f(cx+edge/2,cy+edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy+edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy+edge/2, cz+edge/2);
	glEnd();

	glBegin(GL_POLYGON);  // face 6 dir -y
		glVertex3f(cx+edge/2,cy-edge/2, cz+edge/2);
		glVertex3f(cx+edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz-edge/2);
		glVertex3f(cx-edge/2,cy-edge/2, cz+edge/2);
	glEnd();


}	

void drawCBWire(void)
{ 
	float T, R, edge;
	int u,v,w;
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0, 0.0, -view_distance);
	glRotatef(x_angle, 1.0, 0.0, 0.0);
	glRotatef(y_angle, 0.0, 1.0, 0.0);
	glRotatef(z_angle, 0.0, 0.0, 1.0);


	glClearColor(1.0,1.0,1.0, 0.0);
	
	T=20;

	glLineWidth(2.0);	

	glBegin(GL_LINES);
	glColor3f(0.3, 0.0, 0.0);
		glVertex3f(-T-3,0.0,0.0);
		glVertex3f(T+3, 0.0, 0.0);
		glColor3f(0.0, 0.3, 0.0);
		glVertex3f(0.0, -T-3,0.0);
		glVertex3f(0.0, T+3, 0.0);
		glColor3f(0.0, 0.0, 0.3);
		glVertex3f(0.0,0.0, -T-3.0);
		glVertex3f(0.0, 0.0, T+3);

	glEnd();


	R=5;
	edge=(2*R)/sqrt3;
	w=0;v=0;u=0;
	glPointSize(5.0);
    glColor3f(0.0, 0.0, 0.0);
    glBegin(GL_POINTS);
	for (w=-2;w<3;w++)
		for (u=-2;u<3;u++)
			for (v=-2;v<3;v++)
					glVertex3f(u*edge,v*edge,w*edge);
	glEnd();

	for (w=-2;w<3;w++)
		for (u=-2;u<3;u++)
			for (v=-2;v<3;v++)
					draw_Cube_Wire(u*edge,v*edge,w*edge,edge);
			
	glFlush();
	glutSwapBuffers();
}

void drawCBSolid(void)
{
	float T, R, edge;
	int u,v,w;
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0, 0.0, -view_distance);
	glRotatef(x_angle, 1.0, 0.0, 0.0);
	glRotatef(y_angle, 0.0, 1.0, 0.0);
	glRotatef(z_angle, 0.0, 0.0, 1.0);


	glClearColor(1.0,1.0,1.0, 0.0);
	
	T=20;

	R=5;
	edge=(2*R)/sqrt3;
	for (w=-2;w<3;w++)
		for (u=-2;u<3;u++)
			for (v=-2;v<3;v++)
					draw_Cube(u*edge,v*edge,w*edge,edge);
			
	glFlush();
	glutSwapBuffers();

}
